// Copyright (c) 2014 Tel-Aviv University (Israel).
// All rights reserved.
//
// This file is part of ACG (www.acg.cs.tau.ac.il).
// You can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// Author(s): Michal Kleinbort <balasmic@post.tau.ac.il>
//            Oren Salzman <orenzalz@post.tau.ac.il >          

#ifndef PAIRS_H__RTG
#define PAIRS_H__RTG

namespace RTG
{

    /** \brief An absract class for the Pairs data structure
	**/
	class PairsStruct
    {
     public:

        /** \brief Constructor */
        PairsStruct(unsigned int n, bool add_once = true)
            : num_of_points_(n), add_once_(add_once)
        {}

        /** \brief Destructor */
        virtual ~PairsStruct(void)
        {
            clear();
        }

		/** \brief Check whether a pair exists in the structure */
		virtual bool pair_exists(int i, int j) const = 0;
		
		/** \brief Add pair to the structure */
		virtual void add_pair(int i, int j) = 0;

        /** \brief Output the neighbors of the i-th data point */
		virtual void neighbors_of_i(int i, std::vector<int>& neighborsVec) const = 0;

         /** \brief Output all neighboring pairs */
		virtual void all_pairs(std::vector<std::pair<int, int>>& pairsVec) const = 0; 

		/** \brief Clear the datastructure */
        virtual void clear(void) {}

	 protected:
		/// \brief number of data points
        unsigned int num_of_points_;
        /// \brief A flag indicating whether to a pair (i,j) once or twice (add (i,j) and (j,i))
        bool add_once_;
    }; //PairsStruct

} //namespace RTG



#endif //PAIRS_H__RTG